def docs() -> str:
    return """
Insere uma reunião de atendimento avulso infraestrutura no SIGA para múltiplos participantes.

INSTRUÇÃO PARA O AGENTE IA:
ANTES de executar esta função de inserção:
1. MOSTRE ao usuário TODAS as informações que serão criadas/inseridas no sistema
2. APRESENTE os dados de forma clara: área, descrição da reunião, horário (início e fim), solicitante, participantes (quantidade e matrículas), origem, categoria, equipe, projeto, plaqueta
3. PEÇA confirmação explícita do usuário: "Confirma a criação da reunião? (sim/não)"
4. SÓ EXECUTE esta função se o usuário confirmar explicitamente
5. Se o usuário não confirmar, cancele a operação e informe que foi cancelada

Esta função cria múltiplos registros de atendimento avulso infraestrutura (um para cada participante) 
vinculados a uma reunião. Cada participante é inserido individualmente com auto-commit. A função 
tenta inserir todos os participantes e retorna: lista de participantes inseridos com sucesso e 
lista de participantes que falharam (com matrícula, nome e motivo do erro).

ORIENTAÇÃO PARA SOLICITAÇÕES GENÉRICAS:
Quando o usuário solicitar "criar reunião" SEM especificar o tipo:
1. PRIMEIRO: Perguntar "A reunião é vinculada a uma OS ou é uma reunião avulsa?"
2. Se responder "OS" ou mencionar número de OS: Usar inserir_reuniao_atendimentos_os
3. Se responder "avulsa" ou "não tem OS": Perguntar "É de Sistemas ou Infraestrutura?"
   - Se "Sistemas": Usar inserir_reuniao_atendimento_avulso_sistemas
   - Se "Infraestrutura": Usar ESTA função (inserir_reuniao_atendimento_avulso_infraestrutura)
4. Só depois de definir o tipo: Coletar demais informações (data, descrição, participantes, solicitante)

Esta função é específica para Reuniões Avulso Infraestrutura (área de Infraestrutura). Para outros tipos, use:
- `inserir_reuniao_atendimentos_os` (vinculadas a OS)  
- `inserir_reuniao_atendimento_avulso_sistemas` (área de Sistemas)

ORIENTAÇÃO PARA BUSCA DE PARTICIPANTES:
Se o usuário solicitar "reunião com equipe do gerente X" ou "reunião com equipe Help-Desk":
1. PRIMEIRO chamar: listar_usuarios_equipe_por_gerente (com filtros apropriados)
2. DEPOIS chamar: extrair_matriculas_do_xml (para obter lista de matrículas)
3. FINALMENTE chamar: esta função com a lista de participantes obtida

Args:
    data_inicio: Data e hora de início da reunião. Aceita formatos de data ou palavras-chave como "hoje", "agora", "ontem". Convertido automaticamente.
    data_fim: Data e hora de fim da reunião. Aceita formatos de data ou palavras-chave. Obrigatório para reuniões avulsa.
    matricula_solicitante: Matrícula de quem criou/solicitou a reunião (fixo para todos os registros). Aceita "CURRENT_USER" para usar o usuário logado ou número de matrícula. Não pode estar vazio ou ser zero.
    descricao_reuniao: Descrição detalhada da reunião/pauta
    participantes: Lista de matrículas dos participantes da reunião (ex: ["24142", "20634", "12345"]). Não pode estar vazia.
    origem: Canal/origem do atendimento. Deve ser um dos valores válidos em OrigemAtendimentoAvulsoSistemasType. Defaults to "E-mail".
    categoria: Categoria do atendimento. Deve ser uma das categorias válidas em CategoriasInfraestruturaType. Defaults to "AD - Suporte/Dúvidas/Outros".
    equipe: Equipe responsável. Deve ser uma das equipes válidas em EquipeInfraestruturaType. Defaults to "Help-Desk - Aeroporto".
    projeto: Projeto relacionado. Deve ser um dos valores válidos em ProjetoType. Defaults to "Operação Help Desk".
    plaqueta: Plaqueta do equipamento relacionado ao atendimento. Campo opcional, pode ser None ou string. Defaults to None.

Returns:
    XML formatado contendo status da operação, tipo (reuniao_avulso_infraestrutura), área (Infraestrutura), solicitante, total de participantes inseridos, lista de participantes inseridos com sucesso (matrícula, nome e status), lista de erros (matrícula, nome e motivo do erro) para participantes que falharam, mensagem descritiva e metadados da reunião (origem, categoria, equipe, projeto, plaqueta).

Notes:
    - ÁREA FIXA: Esta função cria atendimentos para ÁREA INFRAESTRUTURA (área=2) exclusivamente
    - MÚLTIPLOS REGISTROS: Cria um registro de atendimento avulso para cada participante na lista
    - AUTO-COMMIT INDIVIDUAL: Cada participante é inserido com auto-commit independente usando trocarConexao()
    - INSERÇÃO INDEPENDENTE: Falha de um participante não afeta inserção dos demais
    - SUCESSO PARCIAL: Insere todos os participantes possíveis. Retorna lista de sucessos E lista de erros (com motivo) separadamente
    - FALHA TOTAL: Operação só falha se NENHUM participante puder ser inserido
    - BUSCA DE NOMES: Backend busca nomes de todos os participantes em uma única query antes da inserção
    - VALIDAÇÃO OBRIGATÓRIA: matricula_solicitante não pode estar vazio ou ser zero, lista de participantes não pode estar vazia
    - SOLICITANTE vs PARTICIPANTES: O solicitante é quem criou a reunião (fixo para todos). Os participantes são os analistas que participaram (variável)
    - ESTRUTURA DE DADOS: Cada registro tem: Solicitante=matricula_solicitante (fixo), Analista=participante (variável)
    - PLAQUETA OPCIONAL: Campo plaqueta pode ser None ou string vazia. Se None, será enviado como string vazia para a API
    - NOMES INCLUÍDOS: Participantes inseridos e erros incluem matrícula e nome completo do colaborador
    - FORMATAÇÃO: Participantes e erros formatados com bullets (•) para melhor legibilidade na resposta
    - CONVERSÃO DE DATAS: Datas são automaticamente convertidas usando converter_data_siga com manter_horas=True
    - NORMALIZAÇÃO: Função realiza validação case-insensitive de origem, categoria, equipe e projeto usando constantes de mapeamento
    - CONSTANTES UTILIZADAS: ORIGEM_TO_NUMBER, CATEGORIA_TO_NUMBER, EQUIPE_INFRAESTRUTURA_TO_NUMBER, PROJETO_TO_NUMBER
    - API KEY: Obtida automaticamente da variável de ambiente AVA_API_KEY
    - TIMEOUT: Requisição HTTP tem timeout de 60 segundos
    - ERRO HTTP: Em caso de falha na requisição, retorna erro interno formatado em XML
    - Esta função é diferente de inserir_atendimento_avulso_infraestrutura que cria atendimento único para um analista
    - Integra-se com listar_usuarios_equipe_por_gerente e extrair_matriculas_do_xml para busca de participantes

"""



def docs() -> str:
    return """
Edita as informações de um atendimento avulso sistema existente no sistema SIGA.

**INSTRUÇÃO PARA O AGENTE IA:**
ANTES de executar esta função de edição:
1. SEMPRE chame primeiro `buscar_informacoes_atendimento_avulso(codigo_atendimento, codigo_analista)`
2. MOSTRE ao usuário os dados atuais vs. dados que serão alterados em formato claro e comparativo
3. PEÇA confirmação explícita do usuário: "Confirma as alterações? (sim/não)"
4. SÓ EXECUTE esta função se o usuário confirmar explicitamente
5. Se o usuário não confirmar, cancele a operação e informe que foi cancelada

Esta função permite atualizar todos os campos de um atendimento avulso sistema existente,
incluindo datas, descrição, tipo, origem, sistema, equipe e projeto.
Realiza validação de todos os campos obrigatórios e conversão automática de datas.

**Funcionalidade de busca automática:**
Se o atendimento não for encontrado nesta função (atendimentos avulso sistemas), a mensagem de erro orientará a buscar nas funções editar_atendimentos_os ou editar_atendimento_avulso_infraestrutura,
permitindo busca automática entre os tipos de atendimento.

Args:
    codigo_atendimento: Código único do atendimento avulso a ser editado
    data_inicio: Data e hora de início do atendimento avulso. Aceita formatos de data ou palavras-chave como "hoje", "agora", "ontem"
    data_fim: Data e hora de fim do atendimento avulso. Aceita formatos de data ou palavras-chave como "hoje", "agora", "ontem"
    matricula_solicitante: Matrícula do usuário que está solicitando o atendimento avulso
    descricao_atendimento: Descrição detalhada do atendimento avulso a ser realizado
    codigo_analista: Matrícula do analista/usuário responsável pelo atendimento avulso
    tipo: Tipo do atendimento avulso, deve ser um dos valores válidos.
    origem: Canal/origem do atendimento avulso, deve ser um dos valores válidos.
    sistema: Sistema relacionado ao atendimento avulso, deve ser um dos valores válidos.
    equipe: Equipe responsável pelo atendimento avulso, deve ser uma das equipes válidas.
    projeto: Projeto relacionado ao atendimento avulso, deve ser um dos valores válidos.

Notes:
    - **ATENÇÃO**: Esta operação modifica permanentemente os dados do atendimento
    - **DIFERENCIAÇÃO DE TIPOS**: Esta função é específica para atendimentos avulso sistemas. Para outros tipos, use as funções específicas
    - Para busca automática: Se não encontrar nesta função, use editar_atendimentos_os ou editar_atendimento_avulso_infraestrutura com os mesmos parâmetros
    - A função realiza validação case-insensitive de todos os campos (tipo, origem, sistema, equipe, projeto)
    - As datas são automaticamente convertidas usando converter_data_siga com manter_horas=True
    - Utiliza as constantes: TIPO_TO_NUMBER_ATENDIMENTO_AVULSO, ORIGEM_TO_NUMBER, SISTEMA_TO_NUMBER, EQUIPE_TO_NUMBER, PROJETO_TO_NUMBER para mapear valores para códigos numéricos
    - Todos os parâmetros enviados são incluídos como atributos no XML de resposta
    - A API key é obtida automaticamente da variável de ambiente AVA_API_KEY
    - Em caso de falha na requisição HTTP, retorna erro interno formatado em XML
    - O resultado da API (1 = sucesso, outros valores = erro) é interpretado automaticamente
    - Campos opcionais como nomeSolicitante, centroCusto, etc. são enviados vazios por padrão
    - Esta função edita atendimentos avulsos independentes, não vinculados a nenhuma OS

"""



"""Este módulo guarda todas as funções do MCP visíveis para o Agente usar"""

from os import getenv
import asyncio
from typing import Any, Literal
import zoneinfo
import datetime
import aiohttp
import ujson
import xml.etree.ElementTree as ET
from siga_mcp import memory
from siga_mcp._types import (
    SituacaoUsuarioType,
    EquipeGeralType,
)
from siga_mcp.decorators import controlar_acesso_matricula
from siga_mcp.domain import HoraMinuto, MeioPeriodo
from siga_mcp.dynamic_constants import constantes_md
from siga_mcp.utils import converter_data_siga, get_package_version
from siga_mcp.xml_builder import XMLBuilder


# Busca pendências de registros SIGA do usuário.
@controlar_acesso_matricula
async def buscar_pendencias_lancamentos_atendimentos(
    *,
    matricula: str | int | Literal["CURRENT_USER"] = "CURRENT_USER",
    dataIni: str,
    dataFim: str,
) -> str:
    async with aiohttp.ClientSession(json_serialize=ujson.dumps) as session:
        async with session.post(
            "https://ava3.uniube.br/ava/api/atendimentosAvulsos/buscarPendenciasRegistroAtendimentosSigaIA/",
            json={
                "matricula": matricula,
                "dataIni": converter_data_siga(dataIni),
                "dataFim": converter_data_siga(dataFim),
                "apiKey": getenv("AVA_API_KEY"),
            },
        ) as response:
            try:
                # Verifica se a requisição HTTP foi bem-sucedida (status 2xx)
                response.raise_for_status()

                # Converte a resposta para JSON, permitindo qualquer content-type
                data = await response.json(content_type=None)

                retorno = XMLBuilder().build_xml(
                    # Usa [] se 'result' não existir ou for None
                    data=data.get("result", []),
                    root_element_name="pendencias_lançamentos",
                    item_element_name="pendencias_lançamentos",
                    root_attributes={"matricula": str(matricula)},
                    custom_attributes={"sistema": "SIGA"},
                )

                return retorno

            except Exception:
                # Captura qualquer outro erro não previsto
                return "Erro ao consultar todas as pendências de registros SIGA do usuário."


# Busca pendências para múltiplas matrículas em paralelo (5-10x mais rápido)
# Ideal para gerentes consultando pendências de toda equipe
# Função separada: PHP não conseguiu implementar array devido à complexidade da SQL
@controlar_acesso_matricula
async def buscar_pendencias_multiplas_matriculas(
    *,
    matriculas: list[str | int],
    dataIni: str,
    dataFim: str,
) -> str:
    """Busca pendências de lançamentos para múltiplas matrículas concorrentemente."""

    if not matriculas:
        return XMLBuilder().build_xml(
            data=[],
            root_element_name="pendencias_lançamentos_multiplas",
            item_element_name="pendencia",
            root_attributes={
                "matricula": "",
                "total_matriculas": "0",
                "processadas_ok": "0",
                "com_erro": "0",
            },
            custom_attributes={"sistema": "SIGA"},
        )

    # Executa todas as requisições concorrentemente, usando a função principal
    tasks = [
        buscar_pendencias_lancamentos_atendimentos(
            matricula=matricula, dataIni=dataIni, dataFim=dataFim
        )
        for matricula in matriculas
    ]

    # Aguarda todas as respostas
    resultados = await asyncio.gather(*tasks, return_exceptions=True)

    # Processa e combina resultados
    pendencias_consolidadas = []
    matriculas_ok = []
    matriculas_erro = []

    for i, resultado in enumerate(resultados):
        matricula_atual = str(matriculas[i])

        if isinstance(resultado, Exception):
            matriculas_erro.append(matricula_atual)
            continue

        if "Erro ao consultar" in resultado:
            matriculas_erro.append(matricula_atual)
            continue

        # Sucesso - parseia o XML para extrair dados completos
        matriculas_ok.append(matricula_atual)

        try:
            # ✅ CORRIGIDO: Parseia o XML retornado da função principal
            root = ET.fromstring(resultado)
            pendencias_reais = []

            # Extrai cada pendência real do XML
            for pendencia_elem in root.findall("pendencias_lançamentos"):
                pendencia_data = {}
                for child in pendencia_elem:
                    pendencia_data[child.tag] = child.text

                # Só adiciona se tem dados válidos
                if pendencia_data:
                    pendencias_reais.append(pendencia_data)

            # ✅ NOVO: Adiciona metadados da matrícula
            resultado_matricula = {
                "matricula": matricula_atual,
                "status": "processado",
                "total_pendencias": len(pendencias_reais),
                "tem_dados": len(pendencias_reais) > 0,
            }

            # ✅ CORRIGIDO: Adiciona pendências como elementos XML estruturados
            if pendencias_reais:
                # Para cada pendência, cria elementos individuais numerados
                for idx, pendencia in enumerate(pendencias_reais):
                    for campo, valor in pendencia.items():
                        # Cria elementos como: pendencia_0_analista, pendencia_0_nome_analista, etc.
                        resultado_matricula[f"pendencia_{idx}_{campo}"] = valor

            pendencias_consolidadas.append(resultado_matricula)

        except ET.ParseError:
            # Se não conseguiu parsear o XML, trata como erro
            matriculas_erro.append(matricula_atual)
            if matricula_atual in matriculas_ok:
                matriculas_ok.remove(matricula_atual)

    # ✅ CORRIGIDO: Para matrículas sem pendências, adiciona resultado básico
    for matricula in matriculas_ok:
        # Verifica se já foi processada (tem pendências)
        ja_processada = any(
            item.get("matricula") == matricula for item in pendencias_consolidadas
        )

        if not ja_processada:
            # Adiciona resultado vazio para matrículas sem pendências
            pendencias_consolidadas.append(
                {
                    "matricula": matricula,
                    "status": "processado",
                    "total_pendencias": 0,
                    "tem_dados": False,
                }
            )

    # Constrói XML consolidado
    return XMLBuilder().build_xml(
        data=pendencias_consolidadas,
        root_element_name="pendencias_lançamentos_multiplas",
        item_element_name="resultado_matricula",
        root_attributes={
            "matricula": ",".join([str(m) for m in matriculas]),
            "total_matriculas": str(len(matriculas)),
            "processadas_ok": str(len(matriculas_ok)),
            "com_erro": str(len(matriculas_erro)),
            "matriculas_ok": ",".join(matriculas_ok),
            "matriculas_erro": ",".join(matriculas_erro),
        },
        custom_attributes={"sistema": "SIGA"},
    )


# Calcula o total de horas trabalhadas do usuário.
@controlar_acesso_matricula
async def listar_horas_trabalhadas(
    *,
    matricula: str | int | list[str | int] | Literal["CURRENT_USER"] = "CURRENT_USER",
    data_inicio: str,
    data_fim: str,
) -> str:
    # Preparar matrícula(s) para envio
    if isinstance(matricula, list):
        matricula_param = matricula  # Lista para DAO
    else:
        matricula_param = matricula  # String/int individual

    async with aiohttp.ClientSession(json_serialize=ujson.dumps) as session:
        async with session.post(
            "https://ava3.uniube.br/ava/api/atendimentosAvulsos/buscarTotalHorasTrabalhadasSigaIA/",
            json={
                "matricula": matricula_param,
                "dataIni": converter_data_siga(data_inicio) if data_inicio else "",
                "dataFim": converter_data_siga(data_fim) if data_fim else "",
                "apiKey": getenv("AVA_API_KEY"),
            },
        ) as response:
            try:
                json = await response.json(content_type=None)
                resultado = json["result"]

                retorno = XMLBuilder().build_xml(
                    data=resultado,
                    root_element_name="atendimentos_avulsos",
                    item_element_name="atendimentos_avulsos",
                    root_attributes={
                        "matricula": ",".join(map(str, matricula))
                        if isinstance(matricula, list)
                        else str(matricula),
                    },
                    custom_attributes={"sistema": "SIGA"},
                )

                return retorno

            except Exception:
                return "Erro ao listar horas trabalhadas."


# Lista todos os usuários responsáveis de acordo com a área informada
# Passar para o arquivo tools e montar o docstring.
# Servirá para quando o usuário pedir a lista de usuários responsáveis da OS
async def listar_usuarios_responsaveis_os_siga(area: int) -> str:
    async with aiohttp.ClientSession(json_serialize=ujson.dumps) as session:
        async with session.post(
            "https://ava3.uniube.br/ava/api/usuarios/buscarUsuarioResponsavelOsSigaIA/",
            json={
                "apiKey": getenv("AVA_API_KEY"),
                "area": area,
            },
        ) as response:
            try:
                json_data: list[Any] | dict[str, Any] = await response.json(
                    content_type=None
                )

                # Ajustar conforme retorno real da API PHP
                data: list[Any] = (
                    json_data
                    if isinstance(json_data, list)
                    else json_data.get("result", json_data)
                )

                retorno = XMLBuilder().build_xml(
                    data=data,
                    root_element_name="usuarios_responsaveis",
                    item_element_name="usuario_responsavel",
                    root_attributes={
                        "area": str(area),
                        "tipo": "Sistemas" if area == 1 else "Infraestrutura",
                    },
                    custom_attributes={"sistema": "SIGA"},
                )

                return retorno

            except Exception:
                return f"Erro ao listar usuários responsáveis área {area}."


# Esta função é chamada primeiramente pelo Agente IA quando precisa listar usuários
# Busca usuários de uma equipe filtrados por gerente responsável, descrição da equipe e situação do usuário
# Função MCP TOOL que serve como interface assíncrona e DELEGA para dynamic_constants
# Função WRAPPER: não contém lógica de negócio, apenas repassa parâmetros
@controlar_acesso_matricula
async def listar_usuarios_equipe_por_gerente(
    matricula_gerente: str | int | Literal["CURRENT_USER"] | None = None,
    descricao_equipe: EquipeGeralType | None = None,
    situacao_usuario: SituacaoUsuarioType | None = None,
) -> str:
    from siga_mcp.dynamic_constants import listar_usuarios_equipe_por_gerente

    return listar_usuarios_equipe_por_gerente(
        matricula_gerente=matricula_gerente,
        descricao_equipe=descricao_equipe,
        situacao_usuario=situacao_usuario,
    )


# Função utilitária: converte XML de equipes em lista de matrículas para busca de horas trabalhadas
async def extrair_matriculas_do_xml(xml_string: str) -> list[str]:
    import xml.etree.ElementTree as ET

    try:
        root = ET.fromstring(xml_string)
        matriculas: list[str] = []

        # Busca todos os elementos usuario_equipe_gerente
        for user_element in root.findall(".//usuario_equipe_gerente"):
            usuario_elem = user_element.find("usuario")
            if usuario_elem is not None and usuario_elem.text:
                matricula = usuario_elem.text.strip()
                if matricula:  # Garante que não adiciona strings vazias
                    matriculas.append(matricula)

        return matriculas

    except ET.ParseError:
        # XML malformado
        return []
    except Exception:
        # Qualquer outro erro
        return []


async def atualizar_tempo_gasto_atendimento(
    codigo_analista: int,
    data_inicio: str | None = None,
) -> str:
    if data_inicio:
        data_inicio = converter_data_siga(data_inicio, manter_horas=True)

    try:
        async with aiohttp.ClientSession(json_serialize=ujson.dumps) as session:
            async with session.post(
                "https://ava3.uniube.br/ava/api/atendimentosAvulsos/atualizaTempoGastoAtendimentoAvulso/",
                json={
                    "apiKey": getenv("AVA_API_KEY"),
                    "analista": int(codigo_analista),
                    "dataIni": converter_data_siga(data_inicio) if data_inicio else "",
                },
            ) as response:
                json_response = await response.json(content_type=None)
                result_data = json_response.get("result")

                # Trata a resposta
                if result_data is None:
                    data_final = [
                        {
                            "status": "erro",
                            "mensagem": "Tempo gasto não atualizado. Favor verificar informações digitadas.",
                        }
                    ]
                elif result_data == 1:
                    data_final = [
                        {
                            "status": "sucesso",
                            "mensagem": "Tempo gasto atualizado com sucesso!",
                        }
                    ]
                else:
                    data_final = [
                        {
                            "status": "erro",
                            "mensagem": "Erro ao atualizar o tempo gasto. Tente novamente.",
                        }
                    ]

                return XMLBuilder().build_xml(
                    data=data_final,
                    root_element_name="atualizacao_tempo_gasto",
                    item_element_name="tempo_gasto",
                    root_attributes={
                        "analista": str(codigo_analista),
                        "dataIni": str(data_inicio),
                    },
                    custom_attributes={"sistema": "SIGA"},
                )

    except Exception as e:
        return XMLBuilder().build_xml(
            data=[
                {
                    "status": "erro",
                    "mensagem": f"Erro interno: {str(e)}. Tente novamente mais tarde.",
                }
            ],
            root_element_name="resultado",
            item_element_name="item",
            custom_attributes={"sistema": "SIGA"},
        )


# @controlar_acesso_matricula
# async def cadastrar_todos_dias_pendentes(
#     colaborador: str | Literal["CURRENT_USER"] = "CURRENT_USER",
#     codigo_os: str | None = None,
#     descricao: str = "debugando codigo",
#     cadastrar_como_atendimento_avulso: bool = True,
# ) -> str:
#     if codigo_os is not None and cadastrar_como_atendimento_avulso:
#         raise ValueError(
#             "Os parâmetros 'codigo_os' e 'cadastrar_como_atendimento_avulso' são mutuamente exclusivos. "
#             "Informe apenas um deles."
#         )

#     pendencias = await buscar_pendencias_lancamentos_atendimentos()

#     raise NotImplementedError("Função ainda não implementada.")


async def atualizar_horarios_colaborador(
    entrada_1: str | None = None,
    saida_1: str | None = None,
    entrada_2: str | None = None,
    saida_2: str | None = None,
) -> str:
    _entrada_1 = memory.periodo.periodos[0].entrada
    if entrada_1:
        _entrada_1 = HoraMinuto.from_string(entrada_1)

    _saida_1 = memory.periodo.periodos[0].saida
    if saida_1:
        _saida_1 = HoraMinuto.from_string(saida_1)

    _entrada_2 = memory.periodo.periodos[1].entrada
    if entrada_2:
        _entrada_2 = HoraMinuto.from_string(entrada_2)

    _saida_2 = memory.periodo.periodos[1].saida
    if saida_2:
        _saida_2 = HoraMinuto.from_string(saida_2)

    primeiro_periodo = MeioPeriodo(
        entrada=_entrada_1 if entrada_1 else memory.periodo.periodos[0].entrada,
        saida=_saida_1 if saida_1 else memory.periodo.periodos[0].saida,
    )

    segundo_periodo = MeioPeriodo(
        entrada=_entrada_2 if entrada_2 else memory.periodo.periodos[1].entrada,
        saida=_saida_2 if saida_2 else memory.periodo.periodos[1].saida,
    )

    memory.periodo.alterar_periodos(novos_periodos=[primeiro_periodo, segundo_periodo])

    return "Horários atualizados com sucesso."


# Retorna as constante
async def constantes() -> str:
    """Retorna constantes do sistema SIGA em formato Markdown."""
    return constantes_md


def __version__() -> str:
    brazil_tz = zoneinfo.ZoneInfo("America/Sao_Paulo")

    # Get the current time in the Brazil time zone
    current_time_brazil = datetime.datetime.now(brazil_tz)
    return f"{get_package_version()} - {current_time_brazil}"



import re
from typing import Any

import siga_mcp.utils as utils
from siga_mcp.utils import converter_data_siga, verificar_permissao_acesso_matricula


def test_converter_data_siga_basic_dates():
    # DD/MM/YYYY
    assert re.match(r"\d{2}/\d{2}/\d{4}$", converter_data_siga("01/02/2024"))
    # ISO
    assert re.match(r"\d{2}/\d{2}/\d{4}$", converter_data_siga("2024-02-01"))


def test_converter_data_siga_keywords():
    # hoje/ontem
    hoje = converter_data_siga("hoje")
    ontem = converter_data_siga("ontem")
    assert re.match(r"\d{2}/\d{2}/\d{4}$", hoje)
    assert re.match(r"\d{2}/\d{2}/\d{4}$", ontem)


def test_converter_data_siga_linguagem_natural():
    # fim de semana -> retorna uma data válida
    ds = converter_data_siga("fim de semana")
    assert re.match(r"\d{2}/\d{2}/\d{4}$", ds)


def test_converter_data_siga_manter_horas():
    ds = converter_data_siga("agora", manter_horas=True)
    assert re.match(r"\d{2}/\d{2}/\d{4} \d{2}:\d{2}:\d{2}$", ds)


def test_verificar_permissao_acesso_matricula_stdio(monkeypatch: Any) -> None:
    # Evita tocar em siga_mcp.constants; patch direto no módulo utils
    monkeypatch.setattr(utils, "MCP_TRANSPORT", "stdio", raising=False)
    monkeypatch.setattr(utils, "MATRICULA_USUARIO_ATUAL", "MINHA_MAT", raising=False)

    assert verificar_permissao_acesso_matricula("MINHA_MAT") is True
    assert verificar_permissao_acesso_matricula("OUTRA_MAT") is False


def test_verificar_permissao_acesso_matricula_http(monkeypatch: Any) -> None:
    # Em http retorna None (sem checagem local)
    monkeypatch.setattr(utils, "MCP_TRANSPORT", "http", raising=False)
    assert verificar_permissao_acesso_matricula("QUALQUER") is None



"""Este módulo guarda todas as funções do MCP visíveis para o Agente usar"""

from os import getenv
from typing import Literal
import aiohttp
import ujson
from siga_mcp._types import (
    TipoAtendimentosOSType,
    TiposAtendimentosOSType,
)
from siga_mcp.dynamic_constants import (
    TYPE_TO_NUMBER,
)

from siga_mcp.decorators import controlar_acesso_matricula
from siga_mcp.utils import converter_data_siga, normalizar_parametro
from siga_mcp.xml_builder import XMLBuilder


async def buscar_informacoes_atendimentos_os(
    codigo_atendimento: int,
    codigo_analista: str | Literal["CURRENT_USER"] = "CURRENT_USER",
) -> str:
    from siga_mcp.tools.atendimentos_avulsos import (
        buscar_informacoes_atendimento_avulso,
    )

    async with aiohttp.ClientSession(json_serialize=ujson.dumps) as session:
        async with session.post(
            "https://ava3.uniube.br/ava/api/atendimentosOs/buscarInfoAtendimentosOsSigaIA/",
            json={
                "apiKey": getenv("AVA_API_KEY"),
                "atendimento": codigo_atendimento,
                "analista": codigo_analista,
            },
        ) as response:
            try:
                json = await response.json(content_type=None)
                retorno = XMLBuilder().build_xml(
                    data=json["result"],
                    root_element_name="info_atendimentos_os",
                    item_element_name="info_atendimentos_os",
                    root_attributes={
                        "atendimento": str(codigo_atendimento),
                        "analista": str(codigo_analista),
                    },
                    custom_attributes={"sistema": "SIGA"},
                )

                return retorno

            except Exception:
                # Se não encontrou em Atendimento OS, tenta buscar em atendimentos avulsos
                try:
                    return await buscar_informacoes_atendimento_avulso(
                        codigo_atendimento, codigo_analista
                    )
                except Exception:
                    return "Erro ao buscar as informações do atendimento em ambas as tabelas (OS e Avulso)."


async def editar_atendimentos_os(
    codigo_atendimento: int,
    codigo_os: int,
    data_inicio: str,
    codigo_analista: int,
    descricao_atendimento: str,
    tipo_atendimento: TiposAtendimentosOSType = "Implementação",
    data_fim: str | None = None,
    primeiro_atendimento: bool = False,
    apresenta_solucao: bool = False,
) -> str:
    if data_inicio:
        data_inicio = converter_data_siga(data_inicio, manter_horas=True)

    if data_fim:
        data_fim = converter_data_siga(data_fim, manter_horas=True)

    # Normaliza parâmetro TIPO_ATENDIMENTO (case-insensitive) e valida se existe na constante
    tipo_final, erro_xml = normalizar_parametro(
        tipo_atendimento,
        TYPE_TO_NUMBER,
        data=[
            {
                "status": "erro",
                "tipo_erro": "tipo_invalido",
                "tipo_informado": tipo_atendimento,
                "mensagem": f"Tipo '{tipo_atendimento}' não encontrado na constante TYPE_TO_NUMBER",
                "tipos_validos": list(TYPE_TO_NUMBER.keys()),
            }
        ],
        root_element_name="erro_validacao",
        item_element_name="erro",
        root_attributes={"sistema": "SIGA", "funcao": "editar_atendimentos_os"},
        custom_attributes={"sistema": "SIGA"},
    )

    if erro_xml:  # Se tipo inválido, retorna erro e para execução
        return erro_xml

    try:
        async with aiohttp.ClientSession(json_serialize=ujson.dumps) as session:
            async with session.post(
                "https://ava3.uniube.br/ava/api/atendimentosOs/updateAtendimentosOsSigaIA/",
                json={
                    "atendimento": codigo_atendimento,
                    "os": codigo_os,
                    "dataIni": data_inicio,
                    "analista": codigo_analista,
                    "descricao": descricao_atendimento,
                    "tipo": tipo_final,
                    "dataFim": data_fim,
                    "primeiroAtendimento": primeiro_atendimento,
                    "apresentaSolucao": apresenta_solucao,
                    "apiKey": getenv("AVA_API_KEY"),
                },
            ) as response:
                json_response = await response.json(content_type=None)
                result_data = json_response.get("result")

                # Trata a resposta
                if result_data is None:
                    data_final = [
                        {
                            "status": "erro",
                            "mensagem": "Atendimento não encontrado em Atendimentos OS. Este código pode estar em atendimento avulso (sistemas ou infraestrutura). Tente buscar nas funções: editar_atendimento_avulso_sistemas ou editar_atendimento_avulso_infraestrutura.",
                        }
                    ]
                elif result_data == 1:
                    data_final = [
                        {
                            "status": "sucesso",
                            "mensagem": "Atendimento editado com sucesso!",
                        }
                    ]
                else:
                    data_final = [
                        {
                            "status": "erro",
                            "mensagem": "Erro ao editar o atendimento. Tente novamente.",
                        }
                    ]

                # Adiciona os root_attributes
                return XMLBuilder().build_xml(
                    data=data_final,
                    root_element_name="ordens_servico",
                    item_element_name="ordem_servico",
                    root_attributes={
                        "os": str(codigo_os),
                        "dataIni": str(data_inicio),
                        "analista": str(codigo_analista),
                        "descricao": str(descricao_atendimento),
                        "tipo": str(tipo_final),
                        "descricao_tipo": str(tipo_atendimento),
                        "dataFim": str(data_fim),
                        "primeiroAtendimento": str(primeiro_atendimento),
                        "apresentaSolucao": str(apresenta_solucao),
                    },
                    custom_attributes={"sistema": "SIGA"},
                )

    except Exception as e:
        return XMLBuilder().build_xml(
            data=[
                {
                    "status": "erro",
                    "mensagem": f"Erro interno: {str(e)}. Tente novamente mais tarde.",
                }
            ],
            root_element_name="resultado",
            item_element_name="item",
            custom_attributes={"sistema": "SIGA"},
        )


async def excluir_atendimentos_os(
    codigo_atendimento: int,
    codigo_analista: str | Literal["CURRENT_USER"] = "CURRENT_USER",
) -> str:
    try:
        async with aiohttp.ClientSession(json_serialize=ujson.dumps) as session:
            async with session.post(
                "https://ava3.uniube.br/ava/api/atendimentosOs/excluiAtendimentosOsSigaIA/",
                json={
                    "apiKey": getenv("AVA_API_KEY"),
                    "atendimento": codigo_atendimento,
                    "analista": codigo_analista,
                },
            ) as response:
                json_response = await response.json(content_type=None)
                result_data = json_response.get("result")

                # Trata a resposta
                if result_data is None:
                    data_final = [
                        {
                            "status": "erro",
                            "mensagem": "Atendimento não encontrado em OS. Tente buscar na função excluir_atendimento_avulso.",
                        }
                    ]
                elif result_data == 1:
                    data_final = [
                        {
                            "status": "sucesso",
                            "mensagem": "Atendimento excluído com sucesso!",
                        }
                    ]
                else:
                    data_final = [
                        {
                            "status": "erro",
                            "mensagem": "Erro ao excluir o atendimento. Tente novamente.",
                        }
                    ]

                # Adiciona os root_attributes
                return XMLBuilder().build_xml(
                    data=data_final,
                    root_element_name="exclusões_atendimento_os",
                    item_element_name="exclusão",
                    root_attributes={
                        "atendimento": str(codigo_atendimento),
                        "analista": str(codigo_analista),
                    },
                    custom_attributes={"sistema": "SIGA"},
                )

    except Exception as e:
        return XMLBuilder().build_xml(
            data=[
                {
                    "status": "erro",
                    "mensagem": f"Erro interno: {str(e)}. Tente novamente mais tarde.",
                }
            ],
            root_element_name="resultado",
            item_element_name="item",
            custom_attributes={"sistema": "SIGA"},
        )


async def inserir_atendimentos_os(
    codigo_os: int,
    data_inicio: str,
    codigo_analista: int,
    descricao_atendimento: str,
    tipo: TipoAtendimentosOSType = "Implementação",
    data_fim: str | None = None,
    primeiro_atendimento: bool = False,
    apresenta_solucao: bool = False,
) -> str:
    # VALIDAÇÃO DO CÓDIGO DO ANALISTA
    if not codigo_analista or codigo_analista <= 0:
        return XMLBuilder().build_xml(
            data=[
                {
                    "status": "erro",
                    "tipo_erro": "analista_obrigatorio",
                    "campo_invalido": "codigo_analista",
                    "valor_informado": str(codigo_analista),
                    "mensagem": f"Campo 'codigo_analista' é obrigatório e deve ser um número maior que zero. Valor informado: {codigo_analista}",
                }
            ],
            root_element_name="erro_validacao",
            item_element_name="erro",
            root_attributes={"sistema": "SIGA", "funcao": "inserir_atendimentos_os"},
            custom_attributes={"sistema": "SIGA"},
        )

    if data_inicio:
        data_inicio = converter_data_siga(data_inicio, manter_horas=True)

    if data_fim:
        data_fim = converter_data_siga(data_fim, manter_horas=True)

    # Normaliza parâmetro TIPO (case-insensitive) e valida se existe na constante
    tipo_final, erro_xml = normalizar_parametro(
        tipo,
        TYPE_TO_NUMBER,
        data=[
            {
                "status": "erro",
                "tipo_erro": "tipo_invalido",
                "tipo_informado": tipo,
                "mensagem": f"Tipo '{tipo}' não encontrado na constante TYPE_TO_NUMBER",
                "tipos_validos": list(TYPE_TO_NUMBER.keys()),
            }
        ],
        root_element_name="erro_validacao",
        item_element_name="erro",
        root_attributes={"sistema": "SIGA", "funcao": "inserir_atendimentos_os"},
        custom_attributes={"sistema": "SIGA"},
    )

    if erro_xml:  # Se tipo inválido, retorna erro e para execução
        return erro_xml

    try:
        async with aiohttp.ClientSession(json_serialize=ujson.dumps) as session:
            async with session.post(
                "https://ava3.uniube.br/ava/api/atendimentosOs/inserirAtendimentosOsSigaIA/",
                json={
                    "os": codigo_os,
                    "dataIni": data_inicio,
                    "analista": codigo_analista,
                    "descricao": descricao_atendimento,
                    "tipo": tipo_final,
                    "dataFim": data_fim,
                    "primeiroAtendimento": primeiro_atendimento,
                    "apresentaSolucao": apresenta_solucao,
                    "apiKey": getenv("AVA_API_KEY"),
                },
            ) as response:
                json_response = await response.json(content_type=None)
                result_data = json_response.get("result")

                # Trata a resposta
                if result_data is None:
                    data_final = [
                        {
                            "status": "erro",
                            "mensagem": "Não foi possível salvar o atendimento. Verifique as informações digitadas.",
                        }
                    ]
                elif result_data == 1:
                    data_final = [
                        {
                            "status": "sucesso",
                            "mensagem": "Atendimento cadastrado com sucesso!",
                        }
                    ]
                else:
                    data_final = [
                        {
                            "status": "erro",
                            "mensagem": "Erro ao gravar o atendimento. Tente novamente.",
                        }
                    ]

                # Adiciona os root_attributes
                return XMLBuilder().build_xml(
                    data=data_final,
                    root_element_name="ordens_servico",
                    item_element_name="ordem_servico",
                    root_attributes={
                        "os": str(codigo_os),
                        "dataIni": str(data_inicio),
                        "analista": str(codigo_analista),
                        "descricao": str(descricao_atendimento),
                        "tipo": str(tipo_final),
                        "descricao_tipo": str(tipo),
                        "dataFim": str(data_fim),
                        "primeiroAtendimento": str(primeiro_atendimento),
                        "apresentaSolucao": str(apresenta_solucao),
                    },
                    custom_attributes={"sistema": "SIGA"},
                )

    except Exception as e:
        return XMLBuilder().build_xml(
            data=[
                {
                    "status": "erro",
                    "mensagem": f"Erro interno: {str(e)}. Tente novamente mais tarde.",
                }
            ],
            root_element_name="resultado",
            item_element_name="item",
            custom_attributes={"sistema": "SIGA"},
        )


@controlar_acesso_matricula
async def listar_atendimentos_os(
    matricula: str | int | Literal["CURRENT_USER"] = "CURRENT_USER",
    codigo_os: str | int | None = None,
    data_inicio: str | None = None,
    data_fim: str | None = None,
) -> str:
    async with aiohttp.ClientSession(json_serialize=ujson.dumps) as session:
        async with session.post(
            "https://ava3.uniube.br/ava/api/atendimentosOs/buscarAtendimentosOsSigaIA/",
            json={
                "matricula": str(matricula),
                "os": str(codigo_os) if codigo_os else "",
                "dataIni": converter_data_siga(data_inicio) if data_inicio else "",
                "dataFim": converter_data_siga(data_fim) if data_fim else "",
                "apiKey": getenv("AVA_API_KEY"),
            },
        ) as response:
            try:
                json = await response.json(content_type=None)
                retorno = XMLBuilder().build_xml(
                    data=json["result"],
                    root_element_name="atendimentos_os",
                    item_element_name="atendimentos_os",
                    root_attributes={
                        "matricula": str(matricula),
                        "os": str(codigo_os) if codigo_os else "",
                        "dataIni": str(data_inicio) if data_inicio else "",
                        "dataFim": str(data_fim) if data_fim else "",
                    },
                    custom_attributes={"sistema": "SIGA"},
                )

                return retorno

            except Exception:
                return "Erro ao listar atendimentos OS."


@controlar_acesso_matricula
async def inserir_reuniao_atendimentos_os(
    codigo_os: int,
    data_inicio: str,
    descricao_reuniao: str,
    participantes: list[str],
    tipo: TipoAtendimentosOSType = "Reunião",
    data_fim: str | None = None,
    primeiro_atendimento: bool | None = None,
    apresenta_solucao: bool | None = None,
) -> str:
    """
    Insere uma reunião de atendimento OS para múltiplos participantes.

    Args:
        codigo_os: Código da OS
        data_inicio: Data e hora de início
        descricao_reuniao: Descrição/pauta da reunião
        participantes: Lista de matrículas dos participantes
        tipo: Tipo do atendimento OS (padrão: "Reunião")
        data_fim: Data e hora de fim (opcional)
        primeiro_atendimento: Flag primeiro atendimento
        apresenta_solucao: Flag apresenta solução

    Returns:
        XML com resultado da operação
    """

    # ========== VALIDAÇÕES ==========

    # Validar lista de participantes
    if not participantes or len(participantes) == 0:
        return XMLBuilder().build_xml(
            data=[
                {
                    "status": "erro",
                    "tipo_erro": "participantes_vazio",
                    "campo_invalido": "participantes",
                    "mensagem": "Lista de participantes está vazia.",
                }
            ],
            root_element_name="erro_validacao",
            item_element_name="erro",
            root_attributes={
                "sistema": "SIGA",
                "funcao": "inserir_reuniao_atendimentos_os",
            },
            custom_attributes={"sistema": "SIGA"},
        )

    # Validar OS
    if not codigo_os or codigo_os <= 0:
        return XMLBuilder().build_xml(
            data=[
                {
                    "status": "erro",
                    "tipo_erro": "os_invalida",
                    "campo_invalido": "codigo_os",
                    "valor_informado": str(codigo_os),
                    "mensagem": f"Código da OS é obrigatório e deve ser maior que zero. Valor informado: {codigo_os}",
                }
            ],
            root_element_name="erro_validacao",
            item_element_name="erro",
            root_attributes={
                "sistema": "SIGA",
                "funcao": "inserir_reuniao_atendimentos_os",
            },
            custom_attributes={"sistema": "SIGA"},
        )

    # ========== CONVERSÃO DE DATAS ==========

    if data_inicio:
        data_inicio = converter_data_siga(data_inicio, manter_horas=True)

    if data_fim:
        data_fim = converter_data_siga(data_fim, manter_horas=True)

    # ========== NORMALIZAÇÃO DE TIPO ==========

    tipo_final, erro_xml = normalizar_parametro(
        tipo,
        TYPE_TO_NUMBER,
        data=[
            {
                "status": "erro",
                "tipo_erro": "tipo_invalido",
                "tipo_informado": tipo,
                "mensagem": f"Tipo '{tipo}' não encontrado na constante TYPE_TO_NUMBER",
                "tipos_validos": list(TYPE_TO_NUMBER.keys()),
            }
        ],
        root_element_name="erro_validacao",
        item_element_name="erro",
        root_attributes={
            "sistema": "SIGA",
            "funcao": "inserir_reuniao_atendimentos_os",
        },
        custom_attributes={"sistema": "SIGA"},
    )

    if erro_xml:
        return erro_xml

    # ========== CONVERSÃO DE FLAGS BOOLEANAS ==========

    primeiro_atendimento_int = None
    if primeiro_atendimento is not None:
        primeiro_atendimento_int = 1 if primeiro_atendimento else 0

    apresenta_solucao_int = None
    if apresenta_solucao is not None:
        apresenta_solucao_int = 1 if apresenta_solucao else 0

    # ========== CHAMADA HTTP ==========

    try:
        async with aiohttp.ClientSession(json_serialize=ujson.dumps) as session:
            async with session.post(
                "https://ava3.uniube.br/ava/api/reunioes/criarReuniaoOS/",
                json={
                    "apiKey": getenv("AVA_API_KEY"),
                    "os": codigo_os,
                    "dataIni": data_inicio,
                    "descricao": descricao_reuniao,
                    "participantes": participantes,
                    "dataFim": data_fim,
                    "primeiroAtendimento": primeiro_atendimento_int,
                    "apresentaSolucao": apresenta_solucao_int,
                },
                timeout=aiohttp.ClientTimeout(total=60),
            ) as response:
                json_response = await response.json(content_type=None)

                if json_response is None:
                    return XMLBuilder().build_xml(
                        data=[
                            {
                                "status": "erro",
                                "mensagem": "API retornou resposta vazia.",
                            }
                        ],
                        root_element_name="resultado",
                        item_element_name="item",
                        custom_attributes={"sistema": "SIGA"},
                    )

                result_data = json_response.get("result", {})

                if not result_data:
                    result_data = json_response

                success = result_data.get("success", False)

                if success:
                    participantes_inseridos = result_data.get(
                        "participantes_inseridos", []
                    )
                    erros = result_data.get("erros", [])
                    total_participantes = result_data.get("total_participantes", 0)

                    # ========================================
                    # Formata participantes com quebras de linha
                    # ========================================
                    participantes_texto = ""
                    if participantes_inseridos:
                        participantes_lista = []
                        for p in participantes_inseridos:
                            matricula = p.get("matricula", "N/A")
                            nome = p.get("nome", "N/A")
                            status = p.get("status", "N/A")
                            participantes_lista.append(
                                f"• Matrícula {matricula} ({nome}): {status}"
                            )
                        participantes_texto = "\n".join(participantes_lista)
                    else:
                        participantes_texto = "Nenhum participante inserido"
                    # ========================================

                    # ========================================
                    # Formata erros com quebras de linha
                    # ========================================
                    erros_texto = ""
                    if erros:
                        erros_lista = []
                        for e in erros:
                            matricula = e.get("matricula", "N/A")
                            nome = e.get("nome", "N/A")
                            erro = e.get("erro", "N/A")
                            erros_lista.append(
                                f"• Matrícula {matricula} ({nome}): {erro}"
                            )
                        erros_texto = "\n".join(erros_lista)
                    else:
                        erros_texto = "Nenhum erro"
                    # ========================================

                    return XMLBuilder().build_xml(
                        data=[
                            {
                                "status": "sucesso",
                                "tipo": result_data.get("tipo", "reuniao_os"),
                                "os": result_data.get("os", codigo_os),
                                "total_participantes": total_participantes,
                                "participantes_inseridos": participantes_texto,  # ✅ STRING com \n
                                "erros": erros_texto,  # ✅ STRING com \n
                                "mensagem": result_data.get(
                                    "message",
                                    f"{total_participantes} participante(s) inserido(s) com sucesso na reunião.",
                                ),
                            }
                        ],
                        root_element_name="resultado_reuniao",
                        item_element_name="reuniao",
                        root_attributes={
                            "os": str(codigo_os),
                            "descricao": descricao_reuniao,
                            "data_inicio": str(data_inicio),
                            "data_fim": str(data_fim) if data_fim else "",
                            "tipo": str(tipo_final),
                            "descricao_tipo": str(tipo),
                            "total_participantes": str(total_participantes),
                        },
                        custom_attributes={"sistema": "SIGA"},
                    )

    except Exception as e:
        return XMLBuilder().build_xml(
            data=[
                {
                    "status": "erro",
                    "mensagem": f"Erro interno: {str(e)}. Tente novamente mais tarde.",
                }
            ],
            root_element_name="resultado",
            item_element_name="item",
            custom_attributes={"sistema": "SIGA"},
        )



def docs() -> str:
    return """
        Lista todos os usuários responsáveis por Ordens de Serviço SIGA de acordo com a área especificada.
        Serve para obter listas tanto para RESPONSAVEL quanto para RESPONSAVEL_ATUAL.

        INSTRUÇÃO PARA O AGENTE IA:
        Para solicitações sobre usuários responsáveis:
        1. SEMPRE pergunte a área se não for especificada: "Qual área? (1) Sistemas ou (2) Infraestrutura?"
        2. Execute com area=1 para área de Sistemas
        3. Execute com area=2 para área de Infraestrutura
        4. Use esta função quando o usuário solicitar:
        - "lista de usuários responsáveis"
        - "quem pode ser responsável"
        - "quem pode ser responsável atual"
        - "responsáveis disponíveis"
        - "analistas da área X"

        REGRA CRÍTICA - NUNCA INVENTAR USUÁRIOS:
        - JAMAIS crie, invente ou sugira usuários que não estejam na lista retornada por esta função
        - SE a busca retornar vazia, informe que não há usuários disponíveis para a área
        - APENAS use usuários que apareçam explicitamente no XML de retorno
        - NÃO sugira matrículas ou nomes que não estejam na resposta real da API
        - SE não houver usuários, oriente o usuário a verificar com o administrador do sistema

        CASOS DE USO:
        - RESPONSAVEL: Lista usuários que podem ser definidos como responsáveis por uma OS
        - RESPONSAVEL_ATUAL: Lista usuários que podem ser definidos como responsáveis atuais por uma OS
        - Ambos usam a mesma lista de usuários habilitados por área

        EXEMPLOS DE SOLICITAÇÕES:
        - "Liste os usuários responsáveis da área de Sistemas"
        - "Quem pode ser responsável por uma OS de Infraestrutura?"
        - "Mostre os analistas disponíveis para Sistemas"
        - "Lista de responsáveis atuais da Infraestrutura"
        - "Usuários que podem assumir uma OS"

        Esta função busca todos os usuários habilitados a serem responsáveis ou responsáveis atuais
        por Ordens de Serviço em uma área específica. É utilizada pelas funções inserir_os_sistemas
        e inserir_os_infraestrutura para validação de usuários.

        Notes: 
            - DUPLA FINALIDADE: Serve para RESPONSAVEL e RESPONSAVEL_ATUAL 
            - INTEGRAÇÃO: Usada por inserir_os_sistemas e inserir_os_infraestrutura 
            - DADOS SIMPLES: Retorna apenas matrícula (USUARIO) e nome (NOME) 
            - VALIDAÇÃO: Os usuários retornados são os mesmos das constantes de validação 
            - SEGURANÇA: NUNCA invente usuários que não estejam na resposta da API 
            - COMPLEMENTAR: Existe também a função obter_usuarios_responsavel para casos específicos 
        
        """



from typing import Any, Protocol, runtime_checkable


@runtime_checkable
class ConfidentialClientProtocol(Protocol):
    """Subset of MSAL ConfidentialClientApplication used here.

    Return type allows None to match msal's untyped stubs and avoid incompatibilities.
    """

    def acquire_token_for_client(
        self,
        scopes: list[str],
        claims_challenge: str | None = None,
        **kwargs: Any,
    ) -> dict[str, Any] | None: ...



def docs() -> str:
    return """
Lista todos os atendimentos avulsos registrados por um usuário em um período específico.

INSTRUÇÃO PARA O AGENTE IA:
Para solicitações GENÉRICAS de listagem de atendimentos:
1. SEMPRE execute AMBAS as funções: `listar_atendimentos_os` E `listar_atendimentos_avulsos`
2. Solicitações genéricas incluem: "liste meus atendimentos", "mostre meus lançamentos", "atendimentos de hoje/ontem/semana", etc.
3. COMBINE os resultados de forma organizada para dar visão completa ao usuário
4. Execute APENAS esta função quando o usuário especificar explicitamente "atendimentos avulsos"
5. Use os mesmos filtros (datas, período) em ambas as funções para consistência

EXEMPLOS DE SOLICITAÇÕES QUE REQUEREM AMBAS AS FUNÇÕES:
- "Liste todos os meus atendimentos de hoje"
- "Mostre meus lançamentos da semana passada"  
- "Quais foram meus atendimentos de ontem?"
- "Atendimentos realizados este mês"

Esta função busca atendimentos avulsos (não vinculados a Ordens de Serviço) realizados
por um analista em um intervalo de datas. Os atendimentos avulsos são atividades
registradas independentemente de OSs específicas.

Args:
    matricula: Matrícula do usuário/analista cujos atendimentos
        avulsos serão listados. Se "CURRENT_USER", busca atendimentos do usuário atual (matrícula do .env). Defaults to "CURRENT_USER".
    data_inicio: Data de início do período de busca.
        Aceita formatos de data ou palavras-chave "hoje" ou "ontem".
        Este parâmetro é obrigatório.
    data_fim: Data de fim do período de busca.
        Aceita formatos de data ou palavras-chave "hoje" ou "ontem".
        Este parâmetro é obrigatório.

Notes:
    - As datas são automaticamente convertidas usando converter_data_siga()
    - A função utiliza a API de atendimentos avulsos do sistema SIGA
    - Atendimentos avulsos são diferentes de atendimentos de OS (Ordens de Serviço)
    - A API key é obtida automaticamente da variável de ambiente AVA_API_KEY
    - Em caso de falha na requisição HTTP ou parsing JSON, retorna mensagem de erro simples
    - O parâmetro matricula usa o tipo Literal["CURRENT_USER"] para permitir valores opcionais
    - Os parâmetros data_inicio e data_fim são obrigatórios (não têm valor padrão)
    - A resposta da API é processada através do XMLBuilder para formatação consistente
"""



def docs() -> str:
    return """
Atualiza o tempo gasto dos atendimentos de um analista em uma data específica no sistema SIGA.

Esta função recalcula automaticamente o tempo total gasto por um analista em todos os seus 
atendimentos (avulsos e OS) em uma data específica. É utilizada principalmente após 
inserções, edições ou exclusões de atendimentos para manter a consistência dos dados.

Funcionalidade:
- Recalcula tempo gasto de TODOS os atendimentos do analista na data especificada
- Atualiza tanto atendimentos avulsos quanto atendimentos de OS
- Se data_inicio não for informada, utiliza a data de ontem por padrão
- Operação atômica: garante consistência dos dados através de transações

Args: 
    codigo_analista (int): Código/matrícula do analista cujo tempo gasto será recalculado. 
        Deve corresponder a um analista válido no sistema SIGA. 
    data_inicio (str | None, optional): Data para recálculo do tempo gasto no formato DD/MM/YYYY. 
        Aceita formatos de data ou palavras-chave como "hoje", "ontem", "agora". Se None ou não informado, utiliza a data de ontem por padrão. Defaults to None.

Notes: 
    - OPERAÇÃO AUTOMÁTICA: Esta função é chamada automaticamente após inserções, edições e exclusões de atendimentos para manter consistência 
    - COMPORTAMENTO PADRÃO: Quando data_inicio não é informada, a procedure SEG.P_CALCULA_TEMPO_GASTO utiliza automaticamente a data de ontem 
    - TRANSAÇÃO SEGURA: Utiliza transações para garantir integridade dos dados 
    - PERFORMANCE: Recalcula apenas os atendimentos da data especificada, não todo o histórico do analista 
    - FLEXIBILIDADE DE DATA: Aceita diversos formatos de data através da função converter_data_siga com manter_horas=True 
    - ABRANGÊNCIA: Atualiza tanto atendimentos avulsos (sistemas e infraestrutura) quanto atendimentos de OS 
    - VALIDAÇÃO: Verifica se o analista existe antes de processar 
    - ATOMICIDADE: Se houver erro durante o recálculo, nenhuma alteração é mantida 
    - API KEY: Utiliza automaticamente a chave da variável de ambiente AVA_API_KEY 
    - TIMEOUT: Configurado para requisições assíncronas com tratamento de timeout 
    - LOGS: Retorna informações detalhadas para facilitar debug e monitoramento 
    - COMPATIBILIDADE: Funciona com todos os tipos de atendimento do sistema SIGA

"""



def docs() -> str:
    return """
Busca Ordens de Serviço (OS) do sistema SIGA com filtros avançados e flexíveis.

Esta função oferece uma interface completa para consulta de OS no sistema SIGA,
permitindo filtros por matrícula, código de OS, status, e período. Suporta consultas
tanto individuais quanto em lote, sendo ideal para relatórios e análises.

Funcionalidades:
- Consulta por matrícula única ou múltiplas matrículas
- Busca por código de OS específico ou múltiplas OS
- Filtros por status predefinidos ou customizados
- Filtro por período (data início e fim)
- Agrupamentos predefinidos para consultas rápidas
- Suporte a linguagem natural para datas
- Validação de parâmetros obrigatórios


Args:
    matricula: Matrícula(s) do(s) usuário(s).
    os: Código(s) da(s) OS para consulta específica.
    filtrar_por: Status para filtrar as OS. Consultas comuns:
        - "Todas OS em Aberto" - agrupamento automático disponível
        - ["Concluída", "Concluída por Encaminhamento", "Concluída por substituição"] - para OS concluídas
        - ["Cancelada-Usuário", "Cancelamento DTD | Arquivado"] - para OS canceladas
        - None - para todas as OS independente do status
        - Lista customizada de status individuais
    data_inicio: Data de início do período de consulta. Aceita formatos de data padrão ou palavras-chave em português.
    data_fim: Data de fim do período de consulta. Aceita formatos de data padrão ou palavras-chave em português.

Note:
    - Pelo menos 'matricula' ou 'os' deve ter valor válido para executar a consulta
    - Requer variável de ambiente AVA_API_KEY configurada
    - A função é assíncrona e deve ser chamada com await
    - Utiliza a função converter_data_siga() para processar datas
    - Suporte a linguagem natural para datas ("hoje", "ontem", "agora")
    - Utiliza aiohttp para requisições HTTP assíncronas
    - O XML é formatado usando a classe XMLBuilder interna
    - Parâmetros são keyword-only (uso obrigatório de nomes dos parâmetros)
    - Para OS finalizadas (concluídas + canceladas), combine as listas acima
    - Use filtrar_por=None para buscar TODAS as OS sem filtro de status

"""



def docs() -> str:
    return """
Insere uma reunião de atendimento OS no sistema SIGA para múltiplos participantes.

INSTRUÇÃO PARA O AGENTE IA:
ANTES de executar esta função de inserção:
1. MOSTRE ao usuário TODAS as informações que serão criadas/inseridas no sistema
2. APRESENTE os dados de forma clara: OS, descrição da reunião, horário (início e fim), participantes (quantidade e matrículas), tipo, flags
3. PEÇA confirmação explícita do usuário: "Confirma a criação da reunião? (sim/não)"
4. SÓ EXECUTE esta função se o usuário confirmar explicitamente
5. Se o usuário não confirmar, cancele a operação e informe que foi cancelada

Esta função cria múltiplos registros de atendimento OS (um para cada participante) vinculados 
a uma reunião. Cada participante é inserido individualmente com auto-commit. A função tenta 
inserir todos os participantes e retorna: lista de participantes inseridos com sucesso e 
lista de participantes que falharam (com matrícula, nome e motivo do erro).

ORIENTAÇÃO PARA SOLICITAÇÕES GENÉRICAS:
Quando o usuário solicitar "criar reunião" SEM especificar o tipo:
1. PRIMEIRO: Perguntar "A reunião é vinculada a uma OS ou é uma reunião avulsa?"
2. Se responder "OS" ou mencionar número de OS: Usar ESTA função (inserir_reuniao_atendimentos_os)
3. Se responder "avulsa" ou "não tem OS": Perguntar "É de Sistemas ou Infraestrutura?"
   - Se "Sistemas": Usar inserir_reuniao_atendimento_avulso_sistemas
   - Se "Infraestrutura": Usar inserir_reuniao_atendimento_avulso_infraestrutura
4. Só depois de definir o tipo: Coletar demais informações (data, descrição, participantes)

Esta função é específica para Reuniões de Atendimento OS (vinculadas a uma OS). Para reuniões avulsa, use:
- `inserir_reuniao_atendimento_avulso_sistemas` (área de Sistemas)  
- `inserir_reuniao_atendimento_avulso_infraestrutura` (área de Infraestrutura)

ORIENTAÇÃO PARA BUSCA DE PARTICIPANTES:
Se o usuário solicitar "reunião com equipe do gerente X" ou "reunião com equipe Desenvolvimento":
1. PRIMEIRO chamar: listar_usuarios_equipe_por_gerente (com filtros apropriados)
2. DEPOIS chamar: extrair_matriculas_do_xml (para obter lista de matrículas)
3. FINALMENTE chamar: esta função com a lista de participantes obtida

Args:
    codigo_os: Código da Ordem de Serviço à qual a reunião será vinculada. Deve ser um número maior que zero.
    data_inicio: Data e hora de início da reunião. Aceita formatos de data ou palavras-chave como "hoje", "agora", "ontem". Convertido automaticamente.
    descricao_reuniao: Descrição detalhada da reunião/pauta
    participantes: Lista de matrículas dos participantes da reunião (ex: ["24142", "20634", "12345"]). Não pode estar vazia.
    tipo: Tipo do atendimento OS. Deve ser um dos valores válidos em TipoAtendimentosOSType. Defaults to "Reunião".
    data_fim: Data e hora de fim da reunião. Aceita formatos de data ou palavras-chave. Se None, não define fim. Defaults to None.
    primeiro_atendimento: Flag indicando se é o primeiro atendimento da OS. Se None, calculado automaticamente pelo backend. Defaults to None.
    apresenta_solucao: Flag indicando se a reunião apresenta solução para a OS. Se None, assume False no backend. Defaults to None.

Returns:
    XML formatado contendo status da operação, total de participantes inseridos, lista de participantes inseridos com sucesso (matrícula, nome e status), lista de erros (matrícula, nome e motivo do erro) para participantes que falharam, mensagem descritiva e metadados da reunião.

Notes:
    - MÚLTIPLOS REGISTROS: Cria um registro de atendimento OS para cada participante na lista
    - AUTO-COMMIT INDIVIDUAL: Cada participante é inserido com auto-commit independente usando trocarConexao()
    - INSERÇÃO INDEPENDENTE: Falha de um participante não afeta inserção dos demais
    - SUCESSO PARCIAL: Insere todos os participantes possíveis. Retorna lista de sucessos E lista de erros (com motivo) separadamente
    - FALHA TOTAL: Operação só falha se NENHUM participante puder ser inserido
    - BUSCA DE NOMES: Backend busca nomes de todos os participantes em uma única query antes da inserção
    - VALIDAÇÃO OBRIGATÓRIA: codigo_os deve ser maior que zero, lista de participantes não pode estar vazia
    - NOMES INCLUÍDOS: Participantes inseridos e erros incluem matrícula e nome completo do colaborador
    - FORMATAÇÃO: Participantes e erros formatados com bullets (•) para melhor legibilidade na resposta
    - PRIMEIRO ATENDIMENTO: Se None, o backend calcula automaticamente verificando se já existe atendimento na OS
    - APRESENTA SOLUÇÃO: Se None, assume False (0) no backend
    - CONVERSÃO DE DATAS: Datas são automaticamente convertidas usando converter_data_siga com manter_horas=True
    - NORMALIZAÇÃO DE TIPO: Função realiza validação case-insensitive do tipo usando TYPE_TO_NUMBER
    - API KEY: Obtida automaticamente da variável de ambiente AVA_API_KEY
    - TIMEOUT: Requisição HTTP tem timeout de 60 segundos
    - ERRO HTTP: Em caso de falha na requisição, retorna erro interno formatado em XML
    - Esta função é diferente de inserir_atendimentos_os que cria atendimento único para um analista
    - Integra-se com listar_usuarios_equipe_por_gerente e extrair_matriculas_do_xml para busca de participantes

"""



from siga_mcp import (
    HoraMinuto,
    DuracaoMinutos,
    MeioPeriodo,
    DiaDeTrabalho,
    PendenciasLancamentos,
)


def test_horaminuto_order_and_format():
    h1 = HoraMinuto.from_string("08:05")
    h2 = HoraMinuto.from_string("08:30")
    assert str(h1) == "08:05"
    assert h1 < h2
    assert h2.minutes_since_midnight() - h1.minutes_since_midnight() == 25


def test_duracao_arithmetic_and_format():
    d1 = DuracaoMinutos(480)
    d2 = DuracaoMinutos(30)
    total = d1 + d2
    assert isinstance(total, DuracaoMinutos)
    assert total.minutes == 510
    assert total.to_hhmm() == "08:30"


def test_meioperiodo_duration():
    mp = MeioPeriodo(HoraMinuto.from_string("08:00"), HoraMinuto.from_string("12:00"))
    assert mp.duracao.minutes == 240


def test_diatrabalho_parse_and_completion():
    dt_mod = __import__("datetime")
    dia = DiaDeTrabalho(
        data=dt_mod.date(2024, 1, 2),
        carga_horaria=DuracaoMinutos(480),
        tempo_registrado=DuracaoMinutos(240),
        batidas=[],
    )
    # completar com jornada padrão usando MeioPeriodo
    jornada = [
        MeioPeriodo(HoraMinuto.from_string("08:00"), HoraMinuto.from_string("12:00")),
        MeioPeriodo(HoraMinuto.from_string("13:00"), HoraMinuto.from_string("17:00")),
    ]
    dia.completar_batidas(jornada)
    assert [str(b) for b in dia.batidas] == ["08:00", "12:00", "13:00", "17:00"]
    assert dia.faltam_batidas() is False


def test_pendencialancamentos_from_xml_minimal():
    # XML alinhado ao formato esperado por from_xml (child pendencias_lançamentos)
    xml = (
        '<?xml version="1.0"?>\n'
        '<pendencias_lançamentos matricula="123" total="1" sistema="SIGA">\n'
        "  <pendencias_lançamentos>\n"
        "    <analista>123</analista>\n"
        "    <nome_analista>Fulano</nome_analista>\n"
        "    <dt_atendimento>02/01/24</dt_atendimento>\n"
        "    <carga_horaria>08:00</carga_horaria>\n"
        "    <tempo_registrado>04:00</tempo_registrado>\n"
        "    <percentual>50%</percentual>\n"
        "    <ponto_registrado>08:00, 12:00</ponto_registrado>\n"
        "  </pendencias_lançamentos>\n"
        "</pendencias_lançamentos>\n"
    )
    pend_all = PendenciasLancamentos.from_xml(xml)
    assert pend_all.total is None or int(pend_all.total.value) == 1
    assert pend_all.matricula is None or str(pend_all.matricula) == "123"
    assert len(pend_all.itens) == 1
    pend = pend_all.itens[0]
    assert str(pend.analista.matricula) == "123"
    assert pend.dia.data.isoformat() == "2024-01-02"
    assert pend.dia.carga_horaria.to_hhmm() == "08:00"
    assert pend.dia.tempo_registrado.to_hhmm() == "04:00"
    # batidas parsed
    assert [str(b) for b in pend.dia.batidas] == ["08:00", "12:00"]


def test_pendencias_multi_items_parsing():
    xml = (
        '<?xml version="1.0"?>\n'
        '<pendencias_lançamentos matricula="999" total="2" sistema="SIGA">\n'
        "  <pendencias_lançamentos>\n"
        "    <analista>999</analista>\n"
        "    <nome_analista>Um</nome_analista>\n"
        "    <dt_atendimento>02/01/24</dt_atendimento>\n"
        "    <carga_horaria>08:00</carga_horaria>\n"
        "    <tempo_registrado>04:00</tempo_registrado>\n"
        "    <percentual>50%</percentual>\n"
        "    <ponto_registrado>08:00, 12:00</ponto_registrado>\n"
        "  </pendencias_lançamentos>\n"
        "  <pendencias_lançamentos>\n"
        "    <analista>999</analista>\n"
        "    <nome_analista>Um</nome_analista>\n"
        "    <dt_atendimento>03/01/24</dt_atendimento>\n"
        "    <carga_horaria>08:00</carga_horaria>\n"
        "    <tempo_registrado>08:00</tempo_registrado>\n"
        "    <percentual>100%</percentual>\n"
        "    <ponto_registrado>08:00, 12:00, 13:00, 17:00</ponto_registrado>\n"
        "  </pendencias_lançamentos>\n"
        "</pendencias_lançamentos>\n"
    )
    allp = PendenciasLancamentos.from_xml(xml)
    assert allp.matricula is None or str(allp.matricula) == "999"
    assert allp.total is None or int(allp.total.value) == 2
    assert len(allp.itens) == 2
    dates = [p.dia.data.isoformat() for p in allp.itens]
    assert dates == ["2024-01-02", "2024-01-03"]



def docs() -> str:
    return """
Calcula e lista o total de horas trabalhadas de um ou múltiplos analistas em um período específico.

Esta função consolida as horas trabalhadas de analista(s) considerando tanto os
atendimentos de Ordens de Serviço (OS) quanto os atendimentos avulsos realizados
no período especificado. Fornece um resumo completo da produtividade do(s) analista(s).
Suporta busca individual ou em lote para otimização de performance.

Args:
    matricula: 
        Matrícula do analista ou lista de matrículas dos analistas cujas horas
        trabalhadas serão calculadas. Se "CURRENT_USER", calcula para o usuário atual
        (matrícula do .env). Para múltiplos analistas, forneça uma lista de matrículas.
        Defaults to "CURRENT_USER".
    
    data_inicio: 
        Data de início do período para cálculo das horas.
        Aceita formatos de data ou palavras-chave "hoje" ou "ontem".
        Este parâmetro é obrigatório.
    
    data_fim: 
        Data de fim do período para cálculo das horas.
        Aceita formatos de data ou palavras-chave "hoje" ou "ontem".
        Este parâmetro é obrigatório.

Notes:
    - **OTIMIZAÇÃO**: Função otimizada para múltiplas matrículas - uma consulta em vez de N consultas
    - **COMPATIBILIDADE**: Mantém total compatibilidade com uso individual (uma matrícula)
    - **PERFORMANCE**: Para múltiplos analistas, utiliza consulta em lote no banco de dados
    - **INTEGRAÇÃO COM EQUIPES**: Use com extrair_matriculas_do_xml para relatórios de equipe completos
    - **FLUXO DE EQUIPES**: listar_usuarios_equipe_por_gerente → extrair_matriculas_do_xml → listar_horas_trabalhadas
    - As datas são automaticamente convertidas usando converter_data_siga() quando fornecidas
    - A função utiliza a API de cálculo de horas trabalhadas do sistema SIGA
    - O cálculo inclui tanto atendimentos de OS quanto atendimentos avulsos
    - A API key é obtida automaticamente da variável de ambiente AVA_API_KEY
    - Em caso de falha na requisição HTTP ou parsing JSON, retorna mensagem de erro simples
    - O parâmetro matricula aceita tanto valores individuais quanto listas
    - Os parâmetros data_inicio e data_fim são obrigatórios (não têm valor padrão)
    - A resposta da API é processada através do XMLBuilder para formatação consistente
    - Esta função é útil para relatórios de produtividade e controle de horas individuais ou por equipe
    - O resultado consolida informações de múltiplas fontes (OS e atendimentos avulsos)
    - **QTD_HORAS**: Formato "HH:MM" representando total de horas e minutos trabalhados
    - **ESCALABILIDADE**: Suporta desde consultas individuais até equipes inteiras em uma única chamada
"""



from textwrap import dedent


def docs() -> str:
    return dedent("""\
        Conclui uma Ordem de Serviço no sistema SIGA.

        **INSTRUÇÃO PARA O AGENTE IA:**
        ANTES de executar esta função:
        1. SEMPRE chame primeiro `buscar_informacoes_os(codigo_os)` para verificar se a OS existe
        2. MOSTRE ao usuário as informações da OS encontrada, incluindo o status atual
        3. PEÇA confirmação explícita do usuário: "Confirma a conclusão da OS {{codigo_os}}? (sim/não)"
        4. SÓ EXECUTE esta função se o usuário confirmar explicitamente
        5. Se o usuário não confirmar, cancele a operação e informe que foi cancelada

        Esta é uma função de conveniência para concluir uma OS. Internamente chama alterar_status_os_siga_ia()
        com status de conclusão para melhor experiência do usuário.

        **ORIENTAÇÃO PARA SOLICITAÇÕES:**
        - "Concluir OS 123" → Usar esta função
        - "Finalizar OS 456" → Usar esta função  
        - "Marcar OS 789 como concluída" → Usar esta função
        - "Fechar OS 321" → Usar esta função

        Args:
            codigo_os:  Código da OS a ser concluída
            tipo_conclusao: Tipo de conclusão

        Notes:
            - **FUNÇÃO DE CONVENIÊNCIA**: Internamente chama alterar_status_os_siga_ia()
            - **UX MELHORADA**: Mais intuitivo para usuários que querem apenas concluir uma OS
            - **VALIDAÇÕES**: Todas as validações são feitas pela função principal alterar_status_os_siga_ia()
            - **MESMO COMPORTAMENTO**: Retorna exatamente o mesmo XML de alterar_status_os_siga_ia()
            - **ATENÇÃO**: Esta operação modifica permanentemente o status da OS para um estado de conclusão
            - **DIFERENCIAÇÃO DE FUNÇÃO**: Esta é uma função específica para conclusão. Para outras alterações de status, use alterar_status_os_siga_ia()
            - A função aceita apenas status de conclusão válidos (Concluída, Concluída por Encaminhamento, Concluída por substituição)
            - A validação de tipo_conclusao é case-insensitive (não diferencia maiúsculas/minúsculas)
            - Esta função funciona para OS de qualquer área (Sistemas ou Infraestrutura)
            - A API key é obtida automaticamente da variável de ambiente AVA_API_KEY
            - Em caso de falha na requisição HTTP, retorna erro interno formatado em XML
        """)



from textwrap import dedent


def docs() -> str:
    return dedent("""\
                Função que atualiza a memória do agente para lembrar qual é a jornada de trabalho do colaborador.
                """)



from textwrap import dedent


def docs() -> str:
    return dedent("""\
        Exclui um atendimento de Ordem de Serviço (OS) específico do sistema SIGA.

        INSTRUÇÃO PARA O AGENTE IA:
        ANTES de executar esta função de exclusão:
        1. SEMPRE chame primeiro `buscar_informacoes_atendimentos_os(codigo_atendimento, codigo_analista)`
        2. MOSTRE ao usuário TODAS as informações do atendimento que será EXCLUÍDO
        3. PEÇA confirmação explícita do usuário: "Confirma a exclusão? (sim/não)"
        4. SÓ EXECUTE esta função se o usuário confirmar explicitamente
        5. Se o usuário não confirmar, cancele a operação e informe que foi cancelada

        Esta função remove permanentemente um atendimento específico vinculado a uma OS no sistema SIGA.
        Utiliza o código do atendimento e código do analista para garantir identificação precisa.
        ATENÇÃO: Esta operação é irreversível.
                  
        ORIENTAÇÃO PARA SOLICITAÇÕES GENÉRICAS:
        Quando o usuário solicitar "excluir um atendimento" sem especificar o tipo:
        1. Primeiro: Tente esta função (excluir_atendimentos_os)
        2. Se não encontrar: Use automaticamente excluir_atendimento_avulso
        3. Busca automática: Use as funções buscar_informacoes_atendimentos_os e buscar_informacoes_atendimento_avulso para identificar o tipo
        4. Confirmação: Sempre mostre as informações do atendimento antes de excluir

        Esta função é específica para Atendimentos OS. Para atendimentos avulsos, use:
        - `excluir_atendimento_avulso` (tanto Sistemas quanto Infraestrutura)
                  
        Funcionalidade de busca automática:
        Se o atendimento não for encontrado nesta função (atendimentos OS), a mensagem de erro orientará a buscar na função excluir_atendimento_avulso, permitindo busca automática entre os tipos de atendimento.

        Funcionalidades:
            - Exclui atendimento OS pelo código e analista responsável
            - Garante precisão na identificação do registro correto (evita ambiguidade entre códigos duplicados)
            - Retorna informações estruturadas em formato XML com status da operação
            - Inclui tratamento de erros para diferentes cenários de falha
            - Utiliza autenticação via API Key do AVA
            - Orienta busca automática em atendimentos avulsos quando não encontra o registro


        Args:
            codigo_atendimento: Código único identificador do atendimento OS. Obrigatório.
                Deve ser um número inteiro válido correspondente a um atendimento existente no sistema SIGA.
            codigo_analista: Matrícula do analista/usuário responsável pelo atendimento OS. Obrigatório. 
                É necessário para garantir a identificação precisa do registro, evitando conflitos com códigos duplicados entre diferentes tipos de atendimento.

        Returns
        Notes:
            - DIFERENCIAÇÃO DE FUNÇÃO: Esta função exclui atendimentos OS. Para atendimentos avulsos, use excluir_atendimento_avulso
            - BUSCA AUTOMÁTICA: Se não encontrar aqui, o sistema deve automaticamente tentar excluir_atendimento_avulso
            - ATENÇÃO: Esta operação é irreversível. Uma vez excluído, o atendimento não pode ser recuperado
            - Para busca automática: Se não encontrar nesta função, use excluir_atendimento_avulso com os mesmos parâmetros
            - Requer variável de ambiente AVA_API_KEY configurada
            - A função é assíncrona e deve ser chamada com await
            - Utiliza aiohttp para requisições HTTP assíncronas
            - O XML é formatado usando a classe XMLBuilder interna
            - Ambos os parâmetros (codigo_atendimento e codigo_analista) são obrigatórios para evitar conflitos com códigos duplicados em diferentes tabelas do sistema
            - Use com extrema cautela em ambientes de produção
        """)



"""Este módulo contém funções que geram constantes dinâmicas a partir da API"""

from typing import Any, Literal
import httpx
from os import getenv

from siga_mcp._types import EquipeGeralType, SituacaoUsuarioType
from siga_mcp.clients import langfuse
from siga_mcp.constants import (
    MATRICULA_USUARIO_ATUAL,
    MCP_TRANSPORT,
    NOME_USUARIO_ATUAL,
)
from siga_mcp.prompt import Prompt
from siga_mcp.utils import converter_data_siga, dict_to_markdown, buscar_constante_siga
from siga_mcp.xml_builder import XMLBuilder


# Esta função é chamada pela MCP tool e gera os dados reais para a CONSTANTE
# Busca usuários de uma equipe filtrados por gerente responsável, descrição da equipe e situação do usuário
# Contém toda a lógica de negócio: validações, normalização, requisições HTTP e processamento
# Função SÍNCRONA que faz o trabalho pesado e retorna XML com os resultados
def listar_usuarios_equipe_por_gerente(
    matricula_gerente: str | int | Literal["CURRENT_USER"] | None = None,
    descricao_equipe: EquipeGeralType | None = None,
    situacao_usuario: SituacaoUsuarioType | None = None,
) -> str:
    import requests

    # VALIDAÇÃO E NORMALIZAÇÃO DA EQUIPE (se fornecida)
    equipe_final = ""
    if descricao_equipe is not None:
        # Busca a equipe correta na constante EQUIPE_GERAL_TO_NUMBER ignorando maiúsculas/minúsculas
        equipe_normalizada = next(
            (
                key
                for key in EQUIPE_GERAL_TO_NUMBER.keys()
                if str(key).lower() == str(descricao_equipe).lower()
            ),
            None,
        )

        if equipe_normalizada is None:
            return XMLBuilder().build_xml(
                data=[
                    {
                        "status": "erro",
                        "tipo_erro": "equipe_invalida",
                        "equipe_informada": descricao_equipe,
                        "mensagem": f"Equipe '{descricao_equipe}' não encontrada na constante EQUIPE_GERAL_TO_NUMBER",
                        "equipes_validas": list(EQUIPE_GERAL_TO_NUMBER.keys()),
                    }
                ],
                root_element_name="erro_validacao",
                item_element_name="erro",
                root_attributes={
                    "sistema": "SIGA",
                    "funcao": "listar_usuarios_equipe_por_gerente",
                },
                custom_attributes={"sistema": "SIGA"},
            )

        # NORMALIZA: converte nome → código
        equipe_final = EQUIPE_GERAL_TO_NUMBER[equipe_normalizada]

    # VALIDAÇÃO E NORMALIZAÇÃO DA SITUAÇÃO (se fornecida)
    situacao_final = ""
    if situacao_usuario is not None:
        # Busca a situação correta na constante SITUACAO_USUARIO_TO_NUMBER ignorando maiúsculas/minúsculas
        situacao_normalizada = next(
            (
                key
                for key in SITUACAO_USUARIO_TO_NUMBER.keys()
                if str(key).lower() == str(situacao_usuario).lower()
            ),
            None,
        )

        if situacao_normalizada is None:
            return XMLBuilder().build_xml(
                data=[
                    {
                        "status": "erro",
                        "tipo_erro": "situacao_invalida",
                        "situacao_informada": situacao_usuario,
                        "mensagem": f"Situação '{situacao_usuario}' não encontrada na constante SITUACAO_USUARIO_TO_NUMBER",
                        "situacoes_validas": list(SITUACAO_USUARIO_TO_NUMBER.keys()),
                    }
                ],
                root_element_name="erro_validacao",
                item_element_name="erro",
                root_attributes={
                    "sistema": "SIGA",
                    "funcao": "listar_usuarios_equipe_por_gerente",
                },
                custom_attributes={"sistema": "SIGA"},
            )

        # NORMALIZA: converte nome → número
        situacao_final = SITUACAO_USUARIO_TO_NUMBER[situacao_normalizada]

    try:
        response = requests.post(
            "https://ava3.uniube.br/ava/api/usuarios/listarUsuariosEquipePorGerente/",
            json={
                "apiKey": getenv("AVA_API_KEY"),
                "matriculaGerente": str(matricula_gerente) if matricula_gerente else "",
                "equipe": str(equipe_final) if equipe_final else "",
                "situacaoUsuario": str(situacao_final) if situacao_final else "",
            },
        )

        json_response = response.json()

        # VERIFICAR SE JSON_RESPONSE É None
        if json_response is None:
            return XMLBuilder().build_xml(
                data=[
                    {
                        "status": "erro",
                        "mensagem": "API retornou resposta vazia. Verifique a configuração da API key.",
                    }
                ],
                root_element_name="resultado",
                item_element_name="item",
                custom_attributes={"sistema": "SIGA"},
            )

        result_data = json_response.get("result")

        # Trata a resposta
        if result_data is None or len(result_data) == 0:
            data_final = [
                {
                    "status": "aviso",
                    "mensagem": "Nenhum usuário encontrado para os filtros informados. Verifique se você é gerente de uma equipe ou ajuste os filtros de busca.",
                }
            ]

            # Retorna XML de aviso em vez dos dados
            return XMLBuilder().build_xml(
                data=data_final,
                root_element_name="usuarios_equipe_gerente",
                item_element_name="resultado",
                root_attributes={
                    "matriculaGerente": str(matricula_gerente)
                    if matricula_gerente
                    else "",
                    "equipe": str(equipe_final) if equipe_final else "",
                    "situacaoUsuario": str(situacao_final) if situacao_final else "",
                },
                custom_attributes={"sistema": "SIGA"},
            )
        else:
            # Se há resultados, processar normalmente
            return XMLBuilder().build_xml(
                data=result_data,
                root_element_name="usuarios_equipe_gerente",
                item_element_name="usuario_equipe_gerente",
                root_attributes={
                    "matriculaGerente": str(matricula_gerente)
                    if matricula_gerente
                    else "",
                    "equipe": str(equipe_final) if equipe_final else "",
                    "situacaoUsuario": str(situacao_final) if situacao_final else "",
                },
                custom_attributes={"sistema": "SIGA"},
            )

    except Exception as e:
        return XMLBuilder().build_xml(
            data=[
                {
                    "status": "erro",
                    "mensagem": f"Erro interno: {str(e)}. Tente novamente mais tarde.",
                }
            ],
            root_element_name="resultado",
            item_element_name="item",
            custom_attributes={"sistema": "SIGA"},
        )


# Constante com a lista de colaboradores de todas as equipes da empresa, para ser usado no prompt do sistema, durante o uso do chat do SIGA.
COLABORADORES_PROMPT: str = listar_usuarios_equipe_por_gerente()


# Obter usuário responsável para criação de OS Sistemas e Infraestrutura.
# Usado para montar o Docstring e validação na função, caso o usuário informa matrícula que não está na lista
def obter_usuarios_responsavel(
    area: int,
) -> tuple[str, set[str], list[str], dict[str, int]]:
    """
    Busca usuários responsáveis de uma área no sistema SIGA.

    Args:
        area: Código da área (1 = Sistemas, 2 = Infraestrutura)

    Returns:
        tuple contendo:
        - str: String formatada para docstrings de ferramentas MCP
        - set[str]: Conjunto de IDs para validação rápida O(1)
        - list[str]: Lista formatada para mensagens de erro
        - dict[str, int]: Dicionário {nome: id} para conversão markdown
    """
    nome_area = "Sistemas" if area == 1 else "Infraestrutura"

    try:
        # Fazer requisição HTTP para buscar usuários responsáveis
        with httpx.Client(timeout=60.0) as client:
            response = client.post(
                "https://ava3.uniube.br/ava/api/usuarios/buscarUsuarioResponsavelOsSigaIA/",
                json={
                    "apiKey": getenv("AVA_API_KEY"),
                    "area": area,
                },
            )

            # Extrair dados JSON da resposta
            json_data = response.json()
            data: list[dict[str, Any]] | dict[str, Any] = (  # type: ignore
                json_data
                if isinstance(json_data, list)
                else json_data.get("result", [])
            )

            # Validar se recebeu dados válidos
            if not data or not isinstance(data, list):
                return (
                    f"        - Erro ao carregar usuários responsáveis de {nome_area}",
                    set(),
                    [],
                    {},
                )

            # Remover duplicatas usando dict (chave = ID do usuário)
            usuarios_unicos: dict[str, dict[str, Any]] = {}
            for usuario in data:
                if (
                    isinstance(usuario, dict)  # type: ignore
                    and "USUARIO" in usuario
                    and "NOME" in usuario
                ):
                    usuarios_unicos[usuario["USUARIO"]] = usuario

            # Verificar se encontrou usuários válidos
            if not usuarios_unicos:
                return (
                    f"        - Nenhum usuário responsável encontrado para {nome_area}",
                    set(),
                    [],
                    {},
                )

            # 📝 GERAR LISTA FORMATADA PARA DOCSTRING (ordenada alfabeticamente)
            usuarios_ordenados = sorted(
                usuarios_unicos.values(), key=lambda x: x["NOME"]
            )
            docstring = "\n".join(
                [
                    f'        - "{usuario["NOME"]}" (ID: {usuario["USUARIO"]})'
                    for usuario in usuarios_ordenados
                ]
            )

            # 🔍 GERAR SET DE IDS PARA VALIDAÇÃO RÁPIDA SE O USUÁRIO ESTÁ NA LISTA DE RESPONSÁVEIS DA ÁREA
            ids_validacao = {
                str(usuario["USUARIO"]) for usuario in usuarios_unicos.values()
            }

            # 🔍 GERAR LISTA PARA MENSAGENS DE ERRO, PARA AVISAR QUE O USUÁRIO NÃO ESTÁ NA LISTA DE RESPONSÁVEIS DAQUELA ÁREA
            usuarios_para_erro = [
                f'"{usuario["NOME"]}" (ID: {usuario["USUARIO"]})'
                for usuario in usuarios_ordenados
            ]

            # DICIONÁRIO - Para dict_to_markdown
            usuarios_dict = {
                usuario["NOME"]: int(usuario["USUARIO"])
                for usuario in usuarios_ordenados
            }

            # # Retorna 4 resultados: docstring, set de ids para validação rápida, lista para mensagens de erro e dicionário
            return (docstring, ids_validacao, usuarios_para_erro, usuarios_dict)

    except Exception:
        # Retornar erro em caso de falha na requisição ou processamento
        return (
            f"        - Erro ao carregar usuários responsáveis de {nome_area}",
            set(),
            [],
            {},
        )


# ✅ CONSTANTES CACHED - Executam uma vez quando o módulo é carregado
# 📊 Buscar dados para Sistemas (área 1)
(
    USUARIOS_SISTEMAS_DOCSTRING,
    USUARIOS_SISTEMAS_IDS,
    USUARIOS_SISTEMAS_PARA_ERRO,
    USUARIOS_SISTEMAS_DICT,
) = obter_usuarios_responsavel(1)
# 🔧 Buscar dados para Infraestrutura (área 2)
(
    USUARIOS_INFRAESTRUTURA_DOCSTRING,
    USUARIOS_INFRAESTRUTURA_IDS,
    USUARIOS_INFRAESTRUTURA_PARA_ERRO,
    USUARIOS_INFRAESTRUTURA_DICT,
) = obter_usuarios_responsavel(2)


# EQUIPE_TO_NUMBER (11 equipes: Sistemas + DBA)
EQUIPE_TO_NUMBER = buscar_constante_siga(
    endpoint="https://ava3.uniube.br/ava/api/segequipes/buscarEquipesSigaIA/",
    campo_chave="DESCRICAO_EQUIPE",
    campo_valor="CODIGO_EQUIPE",
    nome_constante="Equipes Sistemas + DBA",
    params={"classe": 1, "equipesIncluir": ["DBA"]},
)

# EQUIPE_INFRAESTRUTURA_TO_NUMBER (22 equipes: Todas de Infraestrutura)
EQUIPE_INFRAESTRUTURA_TO_NUMBER = buscar_constante_siga(
    endpoint="https://ava3.uniube.br/ava/api/segequipes/buscarEquipesSigaIA/",
    campo_chave="DESCRICAO_EQUIPE",
    campo_valor="CODIGO_EQUIPE",
    nome_constante="Equipes Infraestrutura",
    params={"classe": 2},
)

# EQUIPE_GERAL_TO_NUMBER (33 equipes: Todas)
EQUIPE_GERAL_TO_NUMBER = buscar_constante_siga(
    endpoint="https://ava3.uniube.br/ava/api/segequipes/buscarEquipesSigaIA/",
    campo_chave="DESCRICAO_EQUIPE",
    campo_valor="CODIGO_EQUIPE",
    nome_constante="Equipes Geral",
)

# Buca todos os tipos ativos (11 tipos - Todos ativos)
TYPE_TO_NUMBER = buscar_constante_siga(
    endpoint="https://ava3.uniube.br/ava/api/segTiposOs/buscarTiposOsSigaIA/",
    campo_chave="DESCRICAO_TIPO",
    campo_valor="CODIGO_TIPO",
    nome_constante="Type to Number",
    params={"ativo": 1},
)

# Busca os Tipos para Atendimento Avulso Sistemas (3 tipos específicos)
TIPO_TO_NUMBER_ATENDIMENTO_AVULSO = buscar_constante_siga(
    endpoint="https://ava3.uniube.br/ava/api/segTiposOs/buscarTiposOsSigaIA/",
    campo_chave="DESCRICAO_TIPO",
    campo_valor="CODIGO_TIPO",
    nome_constante="Tipo Atendimento Avulso Sistemas",
    params={"tiposIncluir": [1, 10, 19]},
)

# Busca os Tipos para Atendimento Avulso infraestrutura (10 tipos específicos)
TIPO_TO_NUMBER_ATENDIMENTO_AVULSO_INFRAESTRUTURA = buscar_constante_siga(
    endpoint="https://ava3.uniube.br/ava/api/segTiposOs/buscarTiposOsSigaIA/",
    campo_chave="DESCRICAO_TIPO",
    campo_valor="CODIGO_TIPO",
    nome_constante="Tipo Atendimento Avulso Infraestrutura",
    params={"classe": 2},
)

# Busca os Tipos para Atendimentos OS (7 tipos - Sistemas ativos, exceto Reunião e Anexo)
TIPO_TO_NUMBER_OS_SISTEMAS = buscar_constante_siga(
    endpoint="https://ava3.uniube.br/ava/api/segTiposOs/buscarTiposOsSigaIA/",
    campo_chave="DESCRICAO_TIPO",
    campo_valor="CODIGO_TIPO",
    nome_constante="Tipo OS Sistemas",
    params={"classe": 1, "ativo": 1, "tiposExcluir": [4, 12]},
)

# Busca Projetos para OS automática do banco de dados via API
PROJETO_TO_NUMBER = buscar_constante_siga(
    endpoint="https://ava3.uniube.br/ava/api/segProjetos/buscarProjetosOsSigaIA/",
    campo_chave="DESCRICAO_PROJETO",
    campo_valor="CODIGO_PROJETO",
    nome_constante="Projetos OS Sistemas",
)

# Busca Origens para Atendimentos Avulsos (7 origens, exceto "SATIC" e "Siga")
ORIGEM_TO_NUMBER = buscar_constante_siga(
    endpoint="https://ava3.uniube.br/ava/api/segorigens/buscarOrigensSigaIA/",
    campo_chave="DESCRICAO_ORIGEM",
    campo_valor="CODIGO_ORIGEM",
    nome_constante="Origens Atendimentos Avulso",
    params={"origensExcluir": [6, 8]},
)

# Busca Origens para Atendimentos para OS (todos existentes)
ORIGEM_OS_TO_NUMBER = buscar_constante_siga(
    endpoint="https://ava3.uniube.br/ava/api/segorigens/buscarOrigensSigaIA/",
    campo_chave="DESCRICAO_ORIGEM",
    campo_valor="CODIGO_ORIGEM",
    nome_constante="Origens OS",
)

# Busca Listagem para OS automática do banco de dados via API
LINGUAGEM_TO_NUMBER_OS_SISTEMAS = buscar_constante_siga(
    endpoint="https://ava3.uniube.br/ava/api/seglinguagens/buscarLinguagensOsSigaIA/",
    campo_chave="DESCRICAO_LINGUAGEM",
    campo_valor="CODIGO_LINGUAGEM",
    nome_constante="Linguagens OS Sistemas",
)

# Busca Sistemas para OS automática do banco de dados via API
SISTEMA_TO_NUMBER = buscar_constante_siga(
    endpoint="https://ava3.uniube.br/ava/api/segSistemas/buscarSistemasOsSigaIA/",
    campo_chave="DESCRICAO_SISTEMA",
    campo_valor="CODIGO_SISTEMA",
    nome_constante="Sistemas OS Sistemas",
)

# Busca Categorias para OS automática do banco de dados via API
CATEGORIA_TO_NUMBER = buscar_constante_siga(
    endpoint="https://ava3.uniube.br/ava/api/segoscategorias/buscarCategoriasOsSigaIA/",
    campo_chave="DESCRICAO_CATEGORIA",
    campo_valor="CODIGO_CATEGORIA",
    nome_constante="Categorias OS Infraestrutura",
)

# Listagem para a Combo Status para OS.
STATUS_OS_TO_NUMBER = buscar_constante_siga(
    endpoint="https://ava3.uniube.br/ava/api/segstatus/buscarStatusOsSigaIA/",
    campo_chave="DESCRICAO_STATUS",
    campo_valor="CODIGO_STATUS",
    nome_constante="Status OS",
)

# Listagem para a Combo Interna para OS.
OS_INTERNA_OS_TO_NUMBER = {
    "Sim": 1,
    "Não": 0,
}

# Listagem para a Combo Criticidade para OS.
CRITICIDADE_OS_TO_NUMBER = {
    "Nenhuma": 0,
    "Baixa": 1,
    "Média": 2,
    "Alta": 3,
}

# Listagem para a Combo Criticidade para OS.
PRIORIDADE_USUARIO_OS_TO_NUMBER = {
    "Nenhuma": 0,
    "Urgente": 1,
    "Alta": 2,
    "Média": 3,
    "Baixa": 4,
}

# Constante para situação do usuário
SITUACAO_USUARIO_TO_NUMBER = {
    "Bloqueado": 0,
    "Ativo": 1,
    "Bloqueado (Afastamento)": 2,
    "Bloqueado pelo RH (Individual)": 3,
    "Bloqueado por Falta de Justificativa de Ponto (Individual)": 4,
    "Bloqueado Licença sem Remuneração": 5,
}

constantes_md = dict_to_markdown(
    ("tipos do atendimento os", TYPE_TO_NUMBER),
    ("tipos atendimento avulso sistemas", TIPO_TO_NUMBER_ATENDIMENTO_AVULSO),
    (
        "tipos atendimento avulso infraestrutura",
        TIPO_TO_NUMBER_ATENDIMENTO_AVULSO_INFRAESTRUTURA,
    ),
    ("tipos atendimentos sistemas os", TIPO_TO_NUMBER_OS_SISTEMAS),
    ("origens atendimentos avulsos", ORIGEM_TO_NUMBER),
    ("origens os", ORIGEM_OS_TO_NUMBER),
    ("sistemas", SISTEMA_TO_NUMBER),
    ("categorias", CATEGORIA_TO_NUMBER),
    ("equipes sistemas", EQUIPE_TO_NUMBER),
    ("equipes infraestrutura", EQUIPE_INFRAESTRUTURA_TO_NUMBER),
    ("equipe geral", EQUIPE_GERAL_TO_NUMBER),
    ("projetos", PROJETO_TO_NUMBER),
    ("linguagens programação", LINGUAGEM_TO_NUMBER_OS_SISTEMAS),
    ("os interna", OS_INTERNA_OS_TO_NUMBER),
    ("status os", STATUS_OS_TO_NUMBER),
    ("criticidade", CRITICIDADE_OS_TO_NUMBER),
    ("prioridade do usuario os", PRIORIDADE_USUARIO_OS_TO_NUMBER),
    ("situação do usuário os", SITUACAO_USUARIO_TO_NUMBER),
    ("usuarios responsaveis sistemas", USUARIOS_SISTEMAS_DICT),
    ("usuários responsáveis infraestrutura", USUARIOS_INFRAESTRUTURA_DICT),
)

SYSTEM_INSTRUCTIONS: Prompt = Prompt.from_text(
    langfuse.get_prompt("siga.developer").compile()
).compile(constantes=constantes_md)

if MCP_TRANSPORT == "stdio":
    SYSTEM_INSTRUCTIONS.compile(
        colaboradores=COLABORADORES_PROMPT,
        data=converter_data_siga("hoje"),
        nome_usuario=NOME_USUARIO_ATUAL,
        matricula=MATRICULA_USUARIO_ATUAL,
        preferencias_usuario=getenv("USER_PREFERENCES", "Sem preferências definidas."),
        constantes=constantes_md,
    )
